﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml;
using System.Xml.Linq;
using System.Text;
using System.Data.SqlClient;

namespace AdventureWorks.Generator
{
    class RDLGenerator
    {
        public void Run(string[] args)
        { 
            int iUserID = Int32.Parse(args[0]);
            int iReportSetID = Int32.Parse(args[1]);
            string sParentReport = args[2];
            string sReportTitle = args[3];
            string[] sReport = new string[1];
            string[] sReportStyle = new string[1];
            string[] sReportSection = new string [1];
            int rows = 0;

            try
            {

                try
                {
                    rows = GetTOC(iUserID, iReportSetID, ref sReport, ref sReportStyle, ref sReportSection);
                }
                catch (Exception ex)
                {
                    Console.WriteLine("Error on GetTOC():" + ex.Message);
                }
                try
                {
                    BuildParentReport(iUserID, iReportSetID, sParentReport, rows, sReportTitle, ref sReport, ref sReportStyle, ref sReportSection);
                }
                catch (Exception ex)
                {
                    Console.WriteLine("Error on BuildParentReport():" + ex.Message);
                }

                Console.WriteLine("RDL file generated successfully.");
            }
            catch (Exception ex)
            {
                Console.WriteLine("Error: " + ex.Message);
            }
            finally
            {
                Console.WriteLine("Press ENTER to continue.");
                Console.ReadLine();
            }
        }

        public static int GetTOC(int iUser, int iReportSet, ref string[] sReport, ref string[] sReportStyle, ref string[] sReportSection)
    {
           
            string sConnectionString = GetConnectionString();
            SqlConnection sqlCon = new SqlConnection(sConnectionString);
            sqlCon.Open();

            SqlCommand sqlComm = new SqlCommand("SELECT * FROM [ReportSet] WHERE [UserID] = " + 
                iUser.ToString() + 
                " and [ReportSetID] = " + 
                iReportSet.ToString() + 
                " ORDER BY [SortOrder]", sqlCon);
            SqlDataReader ReaderObj = sqlComm.ExecuteReader();


            int j = 0;
            while (ReaderObj.Read())
            {
                if (j >= 1)
                {
                    string[] sReportTemp = new string[sReport.Length + 1];
                    string[] sReportStyleTemp = new string[sReportStyle.Length + 1];
                    string[] sReportSectionTemp = new string [sReportSection.Length + 1];
                    if (sReport != null)
                    {
                        Array.Copy(sReport, sReportTemp, Math.Min(sReport.Length, sReportTemp.Length));
                        sReport = sReportTemp;
                    }
                     if (sReportStyle != null)
                    {
                        Array.Copy(sReportStyle, sReportStyleTemp, Math.Min(sReportStyle.Length, sReportStyleTemp.Length));
                        sReportStyle = sReportStyleTemp;
                    }
                     if (sReport != null)
                    {
                        Array.Copy(sReportSection, sReportSectionTemp, Math.Min(sReportSection.Length, sReportSectionTemp.Length));
                        sReportSection = sReportSectionTemp;
                    }

                    
                }
                
                sReport[j] = (string) ReaderObj[2];
                sReportStyle[j] = (string) ReaderObj[6];
                sReportSection[j] = (string) ReaderObj[8];
                j = j + 1;

            }

            ReaderObj.Close();
            sqlCon.Close();
            return j - 1;

    }

        public static string GetConnectionString()
        {
            string sConnectionString = "Data Source=(local);Initial Catalog=CustomReports;Integrated Security=True";
            return sConnectionString;
        }

        public static void BuildParentReport(int iUser, int iReportSet, string sRpt, int rows, string sReportTitle,
        ref string[] sReport, ref string[] sReportStyle, ref string[] sReportSection)
        {
            XElement xParentRDL = XElement.Load(sRpt);
            XElement xElementParent;
            XElement xCurrentRDL;

            int i = 0;
            int iPage = 1;
            int iSection = 1;
            string dns  = "{" + xParentRDL.GetDefaultNamespace().ToString() + "}";

            xParentRDL.Descendants(dns + "Rectangle").Where(x => (string) x.Attribute("Name").Value == "Rectangle2").Remove();
            xElementParent = xParentRDL.Descendants(dns + "Rectangle").Where(x => (string) x.Attribute("Name").Value == "Rectangle1").First();

            foreach (string rpt in sReport)
            {
                if (sReportStyle[i] == "Section")
                {
                    xCurrentRDL = BuildSectionRDL(dns, iSection, "ReportTitlePage", iUser, iReportSet, 
                        sReportSection[i], sReport[i], i);
                    iSection = iSection + 1;
                }
                else
                {
                    xCurrentRDL = BuildPageRDL(dns, iPage, sReport[i], i);
                    iPage = iPage + 1;
                }

                xElementParent.AddAfterSelf(xCurrentRDL);

                i = i + 1;
            }

            xElementParent = xParentRDL.Descendants(dns + "Textbox").Where(x=> (string) x.Attribute("Name").Value == "Textbox1").First();
            xElementParent.Descendants(dns + "Value").First().ReplaceWith(new XElement(dns + "Value", sReportTitle));

            xElementParent = xParentRDL.Descendants(dns + "ReportParameter").Where(x => (string) x.Attribute("Name").Value == "UserID").First();
            xElementParent.Descendants(dns + "Value").First().ReplaceWith(new XElement(dns + "Value", iUser.ToString()));

            xElementParent = xParentRDL.Descendants(dns + "ReportParameter").Where(x => (string) x.Attribute("Name").Value == "ReportSetID").First();
            xElementParent.Descendants(dns + "Value").First().ReplaceWith(new XElement(dns + "Value", iReportSet.ToString()));

            xParentRDL.Descendants(dns + "Body").First().Element(dns + "Height").ReplaceWith(new XElement(dns + "Height", (5.5 + 0.25 * sReport.Length).ToString() + "in"));

            xParentRDL.Save("C:\\SSRS\\" + sReportTitle + ".rdl" );


        }
        static public XElement BuildPageRDL(string dns, int iAttrID, string sReportName, int row) 
        {
            XElement xPageRDL = new XElement(dns + "Rectangle", 
                    new XAttribute("Name", "PageRectangle" + iAttrID.ToString()), 
                    new XElement(dns + "ReportItems", 
                    new XElement(dns + "Subreport", 
                      new XAttribute("Name", "PageSubreport" + iAttrID.ToString()), 
                      new XElement(dns + "ReportName", sReportName), 
                      new XElement(dns + "Height", "0.25in"), 
                      new XElement(dns + "Width", "1in"), 
                      new XElement(dns + "Style", 
                       new XElement(dns + "Border", 
                        new XElement(dns + "Style", "None") 
                       ) 
                      ) 
                     ) 
                    ), 
                    new XElement(dns + "PageBreak", 
                     new XElement(dns + "BreakLocation", "Start"), 
                     new XElement(dns + "ResetPageNumber", "true") 
                     ), 
                    new XElement(dns + "PageName", 
                     "=Lookup(" + iAttrID.ToString() + ",Fields!PageNumber.Value,Fields!PageLabel.Value, \"dsTOC\")" 
                    ), 
                    new XElement(dns + "KeepTogether", "true"), 
                    new XElement(dns + "Top", (5 + row * 0.25).ToString() + "in"), 
                    new XElement(dns + "Height", "0.25in"), 
                    new XElement(dns + "Width", "1in"), 
                    new XElement(dns + "ZIndex", "3"), 
                    new XElement(dns + "Style", 
                     new XElement(dns + "Border", 
                      new XElement(dns + "Style", "None") 
                     ) 
                    ) 
                );
            return xPageRDL;
        }
        static public XElement BuildSectionRDL(string dns, int iAttrID, string sReportName, int iUser, int iReportSet, 
            string sSection, string sSectionTitle, int row) 
        {
            XElement xSectionRDL = new XElement(dns + "Rectangle", 
                    new XAttribute("Name", "SectionRectangle" + iAttrID.ToString()), 
                    new XElement(dns + "ReportItems", 
                    new XElement(dns + "Subreport", 
                      new XAttribute("Name", "SectionSubreport" + iAttrID.ToString()), 
                      new XElement(dns + "ReportName", sReportName), 
                      new XElement(dns + "Parameters", 
                        new XElement(dns + "Parameter", 
                            new XAttribute("Name", "UserID"), 
                            new XElement(dns + "Value", iUser.ToString())), 
                        new XElement(dns + "Parameter", 
                            new XAttribute("Name", "ReportSetID"), 
                            new XElement(dns + "Value", iReportSet.ToString())), 
                        new XElement(dns + "Parameter", 
                            new XAttribute("Name", "Section"), 
                            new XElement(dns + "Value", sSection)), 
                        new XElement(dns + "Parameter", 
                            new XAttribute("Name", "SectionName"), 
                            new XElement(dns + "Value", sSectionTitle)) 
                        ), 
                      new XElement(dns + "Height", "0.25in"), 
                      new XElement(dns + "Width", "1in"), 
                      new XElement(dns + "Style", 
                       new XElement(dns + "Border", 
                        new XElement(dns + "Style", "None") 
                       ) 
                      ) 
                     ) 
                    ), 
                    new XElement(dns + "PageBreak", 
                     new XElement(dns + "BreakLocation", "Start"), 
                     new XElement(dns + "ResetPageNumber", "true") 
                     ), 
                    new XElement(dns + "PageName", "ReportTitle"), 
                    new XElement(dns + "KeepTogether", "true"), 
                    new XElement(dns + "Top", (5 + row * 0.25).ToString() + "in"), 
                    new XElement(dns + "Height", "0.25in"), 
                    new XElement(dns + "Width", "1in"), 
                    new XElement(dns + "ZIndex", "3"), 
                    new XElement(dns + "Style", 
                     new XElement(dns + "Border", 
                      new XElement(dns + "Style", "None") 
                     ) 
                    ) 
                );

            return xSectionRDL;

        }

    }
}
